#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Color Balance 2 v1.6
# Copyright (c) 2012-2015 John Lakkas
# john_lakkas@yahoo.gr
# Licence: GNU GPLv3  ( http://www.gnu.org/licenses/ )

# This program is distributed WITHOUT ANY WARRANTY.
# You using it in your own risk.

# Version: 1.6
# > Optionally use small preview for near instant redraw (idea by jontait2, gimpchat.com)
# > Makes headless mode usable
# Version: 1.5.1
# > Minor UI fixes (by jontait2, gimpchat.com)
# Version: 1.5
# > Instant Preview (by jontait2, gimpchat.com)
# Version: 1.4.1
# > Menu repositioning
# Version: 1.4
# > Major Engine refactoring
# Version: 1.3
# > Engine bugfix
# Version: 1.2
# > Midtones bugfix
# Version: 1.1
# > Midtones bugfix
# Version: 1.0.1
# > Description correction
# Version: 1
# > Midtones added
# > Speed enhancement
# Version: 0.3
# > preview version optional
# Version: 0.2.1
# > plug-in finishing
# Version: 0.2
# > bug fix
# Version: 0.1
# > first production version

# Preview function taken from:
# python High Pass Sharpen
# version-1.2 01/04/2010
# Copyright (c) 2008-2010 Paul Sherman
# inspired by layerfx.py by Jonathan Stipe
# and the high-pass-sharpen.scm by Martin Egger

# From the following links:
# http://www.gimphelp.org/python_fu.shtml
# http://www.gimphelp.org/scripts/high_end_sharpen_py.shtml

import gimp, gimpplugin, math
from gimpenums import *
pdb = gimp.pdb
import gtk, gimpui, gimpcolor
from gimpshelf import shelf

class reuse_init(object):
	previewLayer = None

	def get_layer_pos(self, layer):
		i = 0
		while i < len(self.img.layers):
			if layer == self.img.layers[i]:
				return i
		else:
			i += 1
		return -1


	def layer_exists(self, layer):
		return layer != None and layer in self.img.layers

	def removePreviews(self):
		if self.layer_exists(self.previewLayer):
			self.img.remove_layer(self.previewLayer)
			self.previewLayer = None
		gimp.displays_flush()

	def make_label(self, text):
		label = gtk.Label(text)
		label.set_use_underline(True)
		label.set_alignment(1.0, 0.5)
		label.show()
		return label

	def make_slider_and_spinner(self, init, min, max, step, page, digits):
		controls = {'adj':gtk.Adjustment(init, min, max, step, page), 'slider':gtk.HScale(), 'spinner':gtk.SpinButton()}
		controls['slider'].set_adjustment(controls['adj'])
		controls['slider'].set_draw_value(False)
		controls['spinner'].set_adjustment(controls['adj'])
		controls['spinner'].set_digits(digits)
		controls['slider'].show()
		controls['spinner'].show()
		return controls

	def show_error_msg(self, msg):
		origMsgHandler = pdb.gimp_message_get_handler()
		pdb.gimp_message_set_handler(ERROR_CONSOLE)
		pdb.gimp_message(msg)
		pdb.gimp_message_set_handler(origMsgHandler)

	def stringToColor(self, string):
		colorlist = string[5:-1].split(", ")
		return gimpcolor.RGB(float(colorlist[0]), float(colorlist[1]), float(colorlist[2]), float(colorlist[3]))


class color_balance_2(reuse_init):
	def __init__(self, runmode, img, drawable, crsh, mgsh, ybsh, crmid, mgmid, ybmid, crhigh, mghigh, ybhigh):
		self.img = img
		self.drawableor = drawable
		
		if (runmode ==0 ):
			pdb.gimp_image_undo_freeze(self.img)
			self.drawable = pdb.gimp_layer_copy(self.drawableor, True)
			pdb.gimp_image_add_layer(img, self.drawable, -1)
			self.drawable.name = self.drawableor.name + " Color Balance 2 Preview"
			w = pdb.gimp_drawable_width(self.drawableor)
			h = pdb.gimp_drawable_height(self.drawableor)
			if (w>1000 or h>1000):
				if (w>h):
					w2=500
					h2=int(500.*h/w)
				else:
					h2=500
					w2=int(500.*w/h)
				pdb.gimp_layer_scale(self.drawable, w2, h2, True)
			pdb.gimp_item_set_visible(self.drawableor, False)
			pdb.gimp_image_undo_thaw(self.img)
			gimp.displays_flush()
		
			#self.shelfkey = 'layerfx-drop-shadow'
			#print "runmode:"
			#print runmode
			#if (runmode ==0 ):
			self.showDialog()
		else:
			fxlayer = self.makeLayer(self.img, self.drawableor, crsh, mgsh, ybsh, crmid, mgmid, ybmid, crhigh, mghigh, ybhigh, True)


	def showDialog(self):
		self.dialog = gimpui.Dialog("Color Balance 2", "colorbalance2dialog")
		self.dialog.set_transient()

		self.table = gtk.Table(9, 2, True)
		self.table.set_homogeneous(False)
		self.table.set_row_spacings(8)
		self.table.set_col_spacings(8)
		self.table.show()

		#self.distance_label = self.make_label("Strength:")
		#self.table.attach(self.distance_label, 0, 1, 1, 2)
		self.crsh_label = self.make_label("Cyan/Red shadows:")
		self.mgsh_label = self.make_label("Magenta/Green shadows:")
		self.ybsh_label = self.make_label("Yellow/Blue shadows:")
		self.crmid_label = self.make_label("Cyan/Red midtones:")
		self.mgmid_label = self.make_label("Magenta/Green midtones:")
		self.ybmid_label = self.make_label("Yellow/Blue midtones:")
		self.crhigh_label = self.make_label("Cyan/Red highlights:")
		self.mghigh_label = self.make_label("Magenta/Green highlights:")
		self.ybhigh_label = self.make_label("Yellow/Blue highlights:")
		self.table.attach(self.crsh_label, 0, 1, 1, 2)
		self.table.attach(self.mgsh_label, 0, 1, 2, 3)
		self.table.attach(self.ybsh_label, 0, 1, 3, 4)
		self.table.attach(self.crmid_label, 0, 1, 4, 5)
		self.table.attach(self.mgmid_label, 0, 1, 5, 6)
		self.table.attach(self.ybmid_label, 0, 1, 6, 7)
		self.table.attach(self.crhigh_label, 0, 1, 7, 8)
		self.table.attach(self.mghigh_label, 0, 1, 8, 9)
		self.table.attach(self.ybhigh_label, 0, 1, 9, 10)

		#self.strength_spinner = self.make_slider_and_spinner(10.0, 1.0, 20.0, 1.0, 10.0, 1)
		#self.strength_spinner['adj'].set_value(10.0)
		self.crsh_spinner = self.make_slider_and_spinner(0, -100, 100, 1, 1, 0)
		self.mgsh_spinner = self.make_slider_and_spinner(0, -100, 100, 1, 1, 0)
		self.ybsh_spinner = self.make_slider_and_spinner(0, -100, 100, 1, 1, 0)
		self.crmid_spinner = self.make_slider_and_spinner(0, -100, 100, 1, 1, 0)
		self.mgmid_spinner = self.make_slider_and_spinner(0, -100, 100, 1, 1, 0)
		self.ybmid_spinner = self.make_slider_and_spinner(0, -100, 100, 1, 1, 0)
		self.crhigh_spinner = self.make_slider_and_spinner(0, -100, 100, 1, 1, 0)
		self.mghigh_spinner = self.make_slider_and_spinner(0, -100, 100, 1, 1, 0)
		self.ybhigh_spinner = self.make_slider_and_spinner(0, -100, 100, 1, 1, 0)
		self.crsh_spinner['adj'].set_value(0)
		self.mgsh_spinner['adj'].set_value(0)
		self.ybsh_spinner['adj'].set_value(0)
		self.crmid_spinner['adj'].set_value(0)
		self.mgmid_spinner['adj'].set_value(0)
		self.ybmid_spinner['adj'].set_value(0)
		self.crhigh_spinner['adj'].set_value(0)
		self.mghigh_spinner['adj'].set_value(0)
		self.ybhigh_spinner['adj'].set_value(0)


		#self.distance_label.set_mnemonic_widget(self.strength_spinner['spinner'])
		#self.table.attach(self.strength_spinner['slider'], 1, 2, 1, 2)
		#self.table.attach(self.strength_spinner['spinner'], 2, 3, 1, 2)
		self.crsh_label.set_mnemonic_widget(self.crsh_spinner['spinner'])
		self.mgsh_label.set_mnemonic_widget(self.mgsh_spinner['spinner'])
		self.ybsh_label.set_mnemonic_widget(self.ybsh_spinner['spinner'])
		self.crmid_label.set_mnemonic_widget(self.crmid_spinner['spinner'])
		self.mgmid_label.set_mnemonic_widget(self.mgmid_spinner['spinner'])
		self.ybmid_label.set_mnemonic_widget(self.ybmid_spinner['spinner'])
		self.crhigh_label.set_mnemonic_widget(self.crhigh_spinner['spinner'])
		self.mghigh_label.set_mnemonic_widget(self.mghigh_spinner['spinner'])
		self.ybhigh_label.set_mnemonic_widget(self.ybhigh_spinner['spinner'])
		self.table.attach(self.crsh_spinner['slider'], 1, 2, 1, 2)
		self.table.attach(self.mgsh_spinner['slider'], 1, 2, 2, 3)
		self.table.attach(self.ybsh_spinner['slider'], 1, 2, 3, 4)
		self.table.attach(self.crmid_spinner['slider'], 1, 2, 4, 5)
		self.table.attach(self.mgmid_spinner['slider'], 1, 2, 5, 6)
		self.table.attach(self.ybmid_spinner['slider'], 1, 2, 6, 7)
		self.table.attach(self.crhigh_spinner['slider'], 1, 2, 7, 8)
		self.table.attach(self.mghigh_spinner['slider'], 1, 2, 8, 9)
		self.table.attach(self.ybhigh_spinner['slider'], 1, 2, 9, 10)
		self.table.attach(self.crsh_spinner['spinner'], 2, 3, 1, 2)
		self.table.attach(self.mgsh_spinner['spinner'], 2, 3, 2, 3)
		self.table.attach(self.ybsh_spinner['spinner'], 2, 3, 3, 4)
		self.table.attach(self.crmid_spinner['spinner'], 2, 3, 4, 5)
		self.table.attach(self.mgmid_spinner['spinner'], 2, 3, 5, 6)
		self.table.attach(self.ybmid_spinner['spinner'], 2, 3, 6, 7)
		self.table.attach(self.crhigh_spinner['spinner'], 2, 3, 7, 8)
		self.table.attach(self.mghigh_spinner['spinner'], 2, 3, 8, 9)
		self.table.attach(self.ybhigh_spinner['spinner'], 2, 3, 9, 10)

		self.crsh_spinner['spinner'].connect("value-changed", self.preview)
		self.mgsh_spinner['spinner'].connect("value-changed", self.preview)
		self.ybsh_spinner['spinner'].connect("value-changed", self.preview)
		self.crmid_spinner['spinner'].connect("value-changed", self.preview)
		self.mgmid_spinner['spinner'].connect("value-changed", self.preview)
		self.ybmid_spinner['spinner'].connect("value-changed", self.preview)
		self.crhigh_spinner['spinner'].connect("value-changed", self.preview)
		self.mghigh_spinner['spinner'].connect("value-changed", self.preview)
		self.ybhigh_spinner['spinner'].connect("value-changed", self.preview)
		

		#self.flatten_check = gtk.CheckButton("Flatten layer when complete")
		#self.flatten_check.show()
		#self.table.attach(self.flatten_check, 3, 4, 1, 2)

		self.dialog.vbox.hbox1 = gtk.HBox(False, 7)
		self.dialog.vbox.hbox1.show()
		self.dialog.vbox.pack_start(self.dialog.vbox.hbox1, True, True, 7)
		self.dialog.vbox.hbox1.pack_start(self.table, True, True, 7)

		reset_button = gtk.Button("_Reset")
		reset_button.connect("clicked", self.resetbutton)
		reset_button.show()

		self.preview_button = gtk.Button("Preview: ON")
		self.preview_button.connect("clicked", self.preview_toggle)
		self.preview_button.set_size_request(100, -1)
		self.preview_button.show()
		
		self.preview_size_button = gtk.Button("Size: S")
		self.preview_size_button.connect("clicked", self.preview_size_toggle)
		#self.preview_size_button.set_size_request(100, -1)
		self.preview_size_button.show()

		if gtk.alternative_dialog_button_order():
			ok_button = self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
			cancel_button = self.dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
			self.dialog.action_area.add(reset_button)
			self.dialog.action_area.add(self.preview_button)
			self.dialog.action_area.add(self.preview_size_button)
		else:
			self.dialog.action_area.add(self.preview_size_button)
			self.dialog.action_area.add(self.preview_button)
			self.dialog.action_area.add(reset_button)
			cancel_button = self.dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
			ok_button = self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
		ok_button.connect("clicked", self.okbutton)
		cancel_button.connect("clicked", self.cancelbutton)

		self.dialog.show()
		self.dialog.run()
		self.removePreviews()


	def okbutton(self, widget):
		pdb.gimp_image_undo_freeze(self.img)
	
		# remove old preview layer if it exists
		if self.layer_exists(self.previewLayer):
			self.img.remove_layer(self.previewLayer)
			self.previewLayer = None
			
		self.img.remove_layer(self.drawable)
		pdb.gimp_item_set_visible(self.drawableor, True)
		pdb.gimp_image_undo_thaw(self.img)
		
		# then build new layer with current settings to actually impliment...
		fxlayer = self.makeLayer(self.img, self.drawableor, 
			self.crsh_spinner['adj'].get_value(),
			self.mgsh_spinner['adj'].get_value(),
			self.ybsh_spinner['adj'].get_value(),
			self.crmid_spinner['adj'].get_value(),
			self.mgmid_spinner['adj'].get_value(),
			self.ybmid_spinner['adj'].get_value(),
			self.crhigh_spinner['adj'].get_value(),
			self.mghigh_spinner['adj'].get_value(),
			self.ybhigh_spinner['adj'].get_value(),
			True)
			
			
	def cancelbutton(self, widget):
		pdb.gimp_image_undo_freeze(self.img)
		self.img.remove_layer(self.drawable)
		pdb.gimp_item_set_visible(self.drawableor, True)
		pdb.gimp_image_undo_thaw(self.img)


	def resetbutton(self, widget):
		ptxt = self.preview_button.get_label()
		
		if ptxt == "Preview: ON":
			ptxt = "Prev: Wait"
			self.preview_button.set_label(ptxt)
			self.crsh_spinner['adj'].set_value(0)
			self.mgsh_spinner['adj'].set_value(0)
			self.ybsh_spinner['adj'].set_value(0)
			self.crmid_spinner['adj'].set_value(0)
			self.mgmid_spinner['adj'].set_value(0)
			self.ybmid_spinner['adj'].set_value(0)
			self.crhigh_spinner['adj'].set_value(0)
			self.mghigh_spinner['adj'].set_value(0)
			self.ybhigh_spinner['adj'].set_value(0)
			ptxt = "Preview: ON"
			self.preview_button.set_label(ptxt)
			self.preview(widget)
		elif (ptxt == "Preview: OFF"):
			self.crsh_spinner['adj'].set_value(0)
			self.mgsh_spinner['adj'].set_value(0)
			self.ybsh_spinner['adj'].set_value(0)
			self.crmid_spinner['adj'].set_value(0)
			self.mgmid_spinner['adj'].set_value(0)
			self.ybmid_spinner['adj'].set_value(0)
			self.crhigh_spinner['adj'].set_value(0)
			self.mghigh_spinner['adj'].set_value(0)
			self.ybhigh_spinner['adj'].set_value(0)
			


	def preview(self, widget):
		ptxt = self.preview_button.get_label()
		psizetxt = self.preview_size_button.get_label()

		pdb.gimp_image_undo_freeze(self.img)
		
		if self.layer_exists(self.previewLayer):
			self.img.remove_layer(self.previewLayer)
		
		w = pdb.gimp_drawable_width(self.drawableor)
		h = pdb.gimp_drawable_height(self.drawableor)
		if self.layer_exists(self.drawable):
			wp = pdb.gimp_drawable_width(self.drawable)
			hp = pdb.gimp_drawable_height(self.drawable)
		else:
			wp=0
			hp=0
		
		if (psizetxt == "Size: S"):
			if (wp>500 or hp>500) or (wp==0 or hp==0):
				self.img.remove_layer(self.drawable)
				self.drawable = pdb.gimp_layer_copy(self.drawableor, True)
				pdb.gimp_image_add_layer(self.img, self.drawable, -1)
				self.drawable.name = self.drawableor.name + " Color Balance 2 Preview"
				if (w>h):
					w2=500
					h2=int(500.*h/w)
				else:
					h2=500
					w2=int(500.*w/h)
				pdb.gimp_layer_scale(self.drawable, w2, h2, True)
		if (psizetxt == "Size: L"):
			if (wp==500 or hp==500) or (wp==0 or hp==0):
				self.img.remove_layer(self.drawable)
				self.drawable = pdb.gimp_layer_copy(self.drawableor, True)
				pdb.gimp_image_add_layer(self.img, self.drawable, -1)
				self.drawable.name = self.drawableor.name + " Color Balance 2 Preview"
		
		pdb.gimp_item_set_visible(self.drawableor, False)
		pdb.gimp_item_set_visible(self.drawable, True)
		
		if (ptxt == "Preview: ON"):
			#self.previewLayer = self.makeLayer(self.img, self.drawable, self.strength_spinner['adj'].get_value(),1)
			self.previewLayer = self.makeLayer(self.img, self.drawable, 
				self.crsh_spinner['adj'].get_value(),
				self.mgsh_spinner['adj'].get_value(),
				self.ybsh_spinner['adj'].get_value(),
				self.crmid_spinner['adj'].get_value(),
				self.mgmid_spinner['adj'].get_value(),
				self.ybmid_spinner['adj'].get_value(),
				self.crhigh_spinner['adj'].get_value(),
				self.mghigh_spinner['adj'].get_value(),
				self.ybhigh_spinner['adj'].get_value(),
				1)
				
		pdb.gimp_image_undo_thaw(self.img)
		gimp.displays_flush()
		
		
	def preview_toggle(self, widget):
		ptxt = self.preview_button.get_label()
		if ptxt == "Preview: ON":
			ptxt = "Preview: OFF"
		elif ptxt == "Preview: OFF":
			ptxt = "Preview: ON"
		self.preview_button.set_label(ptxt)
		self.preview(widget)
		
		
	def preview_size_toggle(self, widget):
		ptxt = self.preview_size_button.get_label()
		if ptxt == "Size: S":
			ptxt = "Size: L"
		elif ptxt == "Size: L":
			ptxt = "Size: S"
		self.preview_size_button.set_label(ptxt)
		self.preview(widget)


	def makeLayer(self, img, drawable, crsh, mgsh, ybsh, crmid, mgmid, ybmid, crhigh, mghigh, ybhigh, final = False):
		pdb.gimp_image_undo_group_start(img)
		tmpLayer1 = pdb.gimp_layer_copy(drawable, True)
		pdb.gimp_image_add_layer(img, tmpLayer1, -1)
		tmpLayer1.name = drawable.name + " Color Balance 2"

		#===================================================================
		
		channelred=1
		channelgreen=2
		channelblue=3
		lowinred=0
		hiinred=255
		lowingreen=0
		hiingreen=255
		lowinblue=0
		hiinblue=255
		gammared=0.
		gammagreen=0.
		gammablue=0.
		
		lowred = 0
		lowgreen = 0
		lowblue = 0
		hired = 0
		higreen = 0
		hiblue = 0

		#MIDTONES
		if (crmid > 0.):
			gammared = gammared +0.4*crmid/100.
			gammagreen = gammagreen -0.3*crmid/100.
			gammablue = gammablue -0.3*crmid/100.
		if (crmid < 0.):
			gammared = gammared +0.3*crmid/100.
			gammagreen = gammagreen -0.4*crmid/100.
			gammablue = gammablue -0.4*crmid/100.
		if (mgmid > 0.):
			gammared = gammared -0.3*mgmid/100.
			gammagreen = gammagreen +0.4*mgmid/100.
			gammablue = gammablue -0.3*mgmid/100.
		if (mgmid < 0.):
			gammared = gammared -0.4*mgmid/100.
			gammagreen = gammagreen +0.3*mgmid/100.
			gammablue = gammablue -0.4*mgmid/100.
		if (ybmid > 0.):
			gammared = gammared -0.3*ybmid/100.
			gammagreen = gammagreen -0.3*ybmid/100.
			gammablue = gammablue +0.4*ybmid/100.
		if (ybmid < 0.):
			gammared = gammared -0.4*ybmid/100.
			gammagreen = gammagreen -0.4*ybmid/100.
			gammablue = gammablue +0.3*ybmid/100.
		gammared = gammared +1.
		gammagreen = gammagreen +1.
		gammablue = gammablue +1.
		
		hsvtool = HSVbasics()
		
		
		#SHADOWS
		if (crsh>0):
			lowred = lowred + crsh
		else:
			lowgreen = lowgreen - crsh
			lowblue = lowblue - crsh
		
		if (mgsh>0):
			lowgreen = lowgreen + mgsh
		else:
			lowred = lowred - mgsh
			lowblue = lowblue - mgsh
		
		if (ybsh>0):
			lowblue = lowblue + ybsh
		else:
			lowred = lowred - ybsh
			lowgreen = lowgreen - ybsh
		
		crsh = lowred
		mgsh = lowgreen
		ybsh = lowblue
		
		hsv_sh = hsvtool.gimp_rgb_to_hsv4([crsh, mgsh, ybsh])
		if (hsv_sh[0] > 0.5):
			hsvc_sh = [hsv_sh[0]-0.5, hsv_sh[1], hsv_sh[2]]
		else:
			hsvc_sh = [hsv_sh[0]+0.5, hsv_sh[1], hsv_sh[2]]
		rgb_sh = hsvtool.gimp_hsv_to_rgb4(hsvc_sh)
		

		#HIGHLIGHTS
		if (crhigh>0):
			hired = hired + crhigh
		else:
			higreen = higreen - crhigh
			hiblue = hiblue - crhigh
		
		if (mghigh>0):
			higreen = higreen + mghigh
		else:
			hired = hired - mghigh
			hiblue = hiblue - mghigh
		
		if (ybhigh>0):
			hiblue = hiblue + ybhigh
		else:
			hired = hired - ybhigh
			higreen = higreen - ybhigh
		
		crhigh = hired
		mghigh = higreen
		ybhigh = hiblue
		
		#hsv_high = hsvtool.gimp_rgb_to_hsv4([crhigh, mghigh, ybhigh])
		#if (hsv_high[0] > 0.5):
		#	hsvc_high = [hsv_high[0]-0.5, hsv_high[1], hsv_high[2]]
		#else:
		#	hsvc_high = [hsv_high[0]+0.5, hsv_high[1], hsv_high[2]]
		#rgb_high = hsvtool.gimp_hsv_to_rgb4(hsvc_high)

		#rgb_high = [255-rgb_high[0], 255-rgb_high[1], 255-rgb_high[2]]
		rgb_high = [255-crhigh, 255-mghigh, 255-ybhigh]
		
		pdb.gimp_levels(tmpLayer1, channelred, rgb_sh[0], rgb_high[0], gammared, 0, 255)
		pdb.gimp_levels(tmpLayer1, channelgreen, rgb_sh[1], rgb_high[1], gammagreen, 0, 255)
		pdb.gimp_levels(tmpLayer1, channelblue, rgb_sh[2], rgb_high[2], gammablue, 0, 255)
		
		gimp.displays_flush()
		pdb.gimp_image_undo_group_end(img)
		return tmpLayer1


class HSVbasics:



	#LIBGIMP - The GIMP Library
	# * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
	# *
	# * This library is free software; you can redistribute it and/or
	# * modify it under the terms of the GNU Lesser General Public
	# * License as published by the Free Software Foundation; either
	# * version 2 of the License, or (at your option) any later version.
	# *
	# * This library is distributed in the hope that it will be useful,
	# * but WITHOUT ANY WARRANTY; without even the implied warranty of
	# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	# * Library General Public License for more details.
	# *
	# * You should have received a copy of the GNU Lesser General Public
	# * License along with this library; if not, write to the
	# * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	# * Boston, MA 02111-1307, USA.



	# * gimp_rgb_to_hsv4:
	# * @rgb:        RGB triplet, rgb[0] is red channel, rgb[1] is green,
	# *              rgb[2] is blue (0..255)
	# * @hue:        Pointer to hue channel (0..1)
	# * @saturation: Pointer to saturation channel (0..1)
	# * @value:      Pointer to value channel (0..1)


	def gimp_rgb_to_hsv4(self, rgb):

		#double red, green, blue;
		#double h, s, v;
		#double min, max;
		#double delta;

		#double hue;
		#double saturation;
		#double value;

		#double[] hsv;

		red   = rgb[0] / 255.0;
		green = rgb[1] / 255.0;
		blue  = rgb[2] / 255.0;

		h = 0.0; #/* Shut up -Wall */

		if (red > green):
			maxc = max([red,   blue])
			minc = min([green, blue])
		else:
			maxc = max([green, blue])
			minc = min([red,   blue])
	
		v = maxc

		if (maxc != 0.0):
			s = (maxc - minc) / maxc
		else:
			s = 0.0

		if (s == 0.0):
			h = 0.0
		else:
			delta = maxc - minc

			if (delta == 0.0):
				delta = 1.0

			if (red == maxc):
				h = (green - blue) / delta
			elif (green == maxc):
				h = 2 + (blue - red) / delta
			elif (blue == maxc):
				h = 4 + (red - green) / delta

			h /= 6.0

			if (h < 0.0):
				h += 1.0
			elif (h > 1.0):
				h -= 1.0

		hue        = h
		saturation = s
		value      = v

		hsv = [hue, saturation, value]

		return hsv



	# * gimp_hsv_to_rgb4:
	# * @rgb:        RGB triplet, rgb[0] is red channel, rgb[1] is green,
	# *              rgb[2] is blue (0..255)
	# * @hue:        Hue channel (0..1)
	# * @saturation: Saturation channel (0..1)
	# * @value:      Value channel (0..1)

	def gimp_hsv_to_rgb4(self, hsv):
	
		#double h, s, v;
		#double f, p, q, t;

		hue = hsv[0]
		saturation = hsv[1]
		value = hsv[2]

		#double[] rgb;

		if (saturation == 0.0):
			hue = value
			saturation = value
			#value = value
		else:
			h = hue * 6.0
			s = saturation
			v = value

			if (h == 6.0):
				h = 0.0;

			f = h - int(h)
			p = v * (1.0 - s)
			q = v * (1.0 - s * f)
			t = v * (1.0 - s * (1.0 - f))

			if (int(h)==0):
				hue = v
				saturation = t
				value = p

			elif (int(h)==1):
				hue = q
				saturation = v
				value = p

			elif (int(h)==2):
				hue = p
				saturation = v
				value = t

			elif (int(h)==3):
				hue = p
				saturation = q
				value = v

			elif (int(h)==4):
				hue = t
				saturation = p
				value = v

			elif (int(h)==5):
				hue = v
				saturation = p
				value = q


		#rgb = new Double[3];

		rgb = [round(hue * 255.0), round(saturation * 255.0), round(value * 255.0)]

		return rgb;


class pycolorbalance2(gimpplugin.plugin):
	def start(self):
		gimp.main(self.init, self.quit, self.query, self._run)

	def init(self):
		pass

	def quit(self):
		pass

	def query(self):
		authorname = "John Lakkas"
		copyrightname = "John Lakkas"
		menu_location = "<Image>/Colors/Color Balance 2"
		date = "March 2015"
		plug_description = "Color balance control using individual channel levels. Includes contributions from jontait2, gimpchat.com."
		plug_help = "Color balance 2 plug-in alters image/layer shadows, midtones and highlights color balance using individual channel levels."
		plug_params = [
			(PDB_INT32, "run_mode", "Run mode"),
			(PDB_IMAGE, "image", "Input image"),
			(PDB_DRAWABLE, "drawable", "Input drawable"),
			####### 3 params above needed by all scripts using gimpplugin.plugin ######################
			(PDB_FLOAT, "crsh", "Cyan/Red shadows"),
			(PDB_FLOAT, "mgsh", "Magenta/Green shadows"),
			(PDB_FLOAT, "ybsh", "Yellow/Blue shadows"),
			(PDB_FLOAT, "crmid", "Cyan/Red midtones"),
			(PDB_FLOAT, "mgmid", "Magenta/Green midtones"),
			(PDB_FLOAT, "ybmid", "Yellow/Blue midtones"),
			(PDB_FLOAT, "crhigh", "Cyan/Red highlights"),
			(PDB_FLOAT, "mghigh", "Magenta/Green highlights"),
			(PDB_FLOAT, "ybhigh", "Yellow/Blue highlights")
			]

		gimp.install_procedure("py_color_balance_2",
			plug_description,
			plug_help,
			authorname,
			copyrightname,
			date,
			menu_location,
			"RGB*, GRAY*",
			PLUGIN,
			plug_params,
			[])

	def py_color_balance_2(self, runmode, img, drawable, crsh=0, mgsh=0, ybsh=0, crmid=0, mgmid=0, ybmid=0, crhigh=0, mghigh=0, ybhigh=0):
		color_balance_2(runmode, img, drawable, crsh, mgsh, ybsh, crmid, mgmid, ybmid, crhigh, mghigh, ybhigh)

if __name__ == '__main__':
	pycolorbalance2().start()
